/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/
/*! \file backbone.cpp
\brief Backbone/core main file \see Backbone


\author Bartosz Szatkowski <bulislaw@linux.com>
*/

#include "backbone.h"
#include "ConfigGenerator.h"
class ConfigGenerator;
#include <QDebug>

int Backbone::_searchLimit;

// Sadly QtConcurrent::mapped doesn't let me use something like calling method of
// some class with supplied argument; so I have to sin against art and put
// global function and variable so I could supply function with some parameter
QString mappedSearch;
QList<Translation*> mapSearch(CommonDictInterface *dict) {
    if(dict)
        return dict->searchWordList(mappedSearch, Backbone::_searchLimit);
    return QList<Translation*>();
}



/*! Smart pointer (kind of) for translation object

    QtConcurrent::mapped uses collection of data and one function, what I need is
    to map single data object to method calls for multiple objects. TranslationPtr
    is an attempt to store method call as a data -> moreover QtConcurrent allows only for
    methods without any parameters so TranslationPtr is created with Translation
    object -> ready to call toHtml() for supplied Translation.

    Another thing is that QtConcurrent doesn't like pointers in data collection
    so TranslationPtr is a way to hide real translation object (pointer to object)
    */
class TranslationPtr {
    Translation* _tr;
public:
    TranslationPtr(Translation* tr) :_tr(tr) {}

    /*! \return translation text for corresponding Translation object */
    QString toHtml() const {
        QString trans;
        trans = _tr->toXml();
        return trans;
    }
};

void Backbone::init() {

   _dictNum = 0;
   _dir = QDir::homePath() + "/.mdictionary/";
   if(!QDir(_dir).exists())
       QDir().mkdir(_dir);

  _configPath = _dir + "mdictionary.config";
  _pluginPath = "/usr/lib/mdictionary/plugins/";

   //Install default config files
   ConfigGenerator confGen;
   ///confGen.generateCss(_dir + "style.css");
   confGen.generateDefaultConfig(_configPath);

   loadPrefs(_configPath);

   loadPlugins();

   loadDicts(_configPath);

   connect(&_resultWatcher, SIGNAL(finished()), this, SLOT(translationReady()));
   connect(&_xmlResultWatcher, SIGNAL(finished()), this,
           SLOT(xmlTranslationReady()));
   connect(&_bookmarkWatcher, SIGNAL(finished()), this,
           SLOT(bookmarksListReady()));
   connect(&_bookmarkSearchWatcher, SIGNAL(finished()), this,
           SLOT(translationReady()));

   // In common opinion perfect thread count is cores_number+1 (in qt perfect
   // thread count is set to cores number) so iam changin it
   QThreadPool::globalInstance()->setMaxThreadCount(
           QThreadPool::globalInstance()->maxThreadCount()+1);

   _history = new History(_historyLen, this);
}



Backbone::Backbone(QString pluginPath, QString configPath, bool dry,
                   QObject *parent)
    : QObject(parent)
{
    _pluginPath = pluginPath;
    _configPath = configPath;

    dryRun = false;
    if(dry)
        dryRun = true;
    init();
}



Backbone::~Backbone()
{
    QListIterator<CommonDictInterface*> it(_dicts.keys());

    while(it.hasNext())
        delete it.next();

    it = QListIterator<CommonDictInterface*>(_plugins);
    while(it.hasNext())
        delete it.next();

    QHashIterator<QString, Translation*> it2(_result);
    while(it2.hasNext())
        delete it2.next().value();

    delete _history;
}




Backbone::Backbone(const Backbone &b) :QObject(b.parent()) {
    init();
    _dicts = QHash<CommonDictInterface*, bool > (b._dicts);
    _plugins = QList<CommonDictInterface* > (b._plugins);
    _result = QHash<QString, Translation* > (b._result);
    _searchLimit = b.searchLimit();
}




int Backbone::searchLimit() const {
    return _searchLimit;
}



QHash<CommonDictInterface*, bool > Backbone::getDictionaries() {
    return _dicts;
}



QList<CommonDictInterface* > Backbone::getPlugins() {
    return _plugins;
}



History* Backbone::history() {
    return _history;
}



QMultiHash<QString, Translation*> Backbone::result() {
    return _result;
}



void Backbone::stopSearching() {
    if(stopped)
        return;

    foreach(CommonDictInterface* dict, _dicts.keys())
        dict->stop();
    stopped = true;
    _innerXmlResult.cancel();
    _innerResult.cancel();
    Q_EMIT searchCanceled();
}



void Backbone::search(QString word){
    _result.clear();
    mappedSearch = word.toLower();

    stopped = false;

    // When dictFin and bookmarkFin is set to true then translationReady()
    // signal is emitted see translationReady(),
    // so when searching only in one of them, corresponding *Fin is set to false
    // and other to true so program is waiting only for one translation
    dictFin = !_searchDicts;
    bookmarkFin = !_searchBookmarks;

    if(!_searchDicts && !_searchBookmarks) {
        Q_EMIT ready();
        Q_EMIT notify(Notify::Warning, tr("You have to specify where You want "
                "to look for translations"));
    }

    if (_searchDicts) {
        _innerResult = QtConcurrent::mapped(activeDicts(), mapSearch);
        _resultWatcher.setFuture(_innerResult);
    }

    if(_searchBookmarks) {
        _innerBookmarks = QtConcurrent::run(_bookmarks,
                &Bookmarks::searchWordList, word);
        _bookmarkSearchWatcher.setFuture(_innerBookmarks);
    }
}



void Backbone::selectedDictionaries(QList<CommonDictInterface* > activeDicts) {
    foreach(CommonDictInterface* dict, _dicts.keys())
        if(activeDicts.contains(dict))
            _dicts[dict] = 1;
        else
            _dicts[dict] = 0;
    dictUpdated();
 }



void Backbone::addDictionary(CommonDictInterface *dict, bool active) {
    addInternalDictionary(dict,active);
    dictUpdated();
}



void Backbone::addInternalDictionary(CommonDictInterface* dict, bool active) {
    if(dict) {
        dict->setHash(++_dictNum); // Hash must be uniqe in every session but not between
        _dicts[dict] = active;
        connect(dict, SIGNAL(settingsChanged()), this, SLOT(dictUpdated()));
        connect(dict, SIGNAL(notify(Notify::NotifyType,QString)), this,
            SIGNAL(notify(Notify::NotifyType,QString)));
    }
}



 void Backbone::removeDictionary(CommonDictInterface *dict) {
     _dicts.remove(dict);
     if(dict) {
        dict->clean();
        delete dict;
     }
     dictUpdated();
 }



 void Backbone::quit() {
    stopSearching();
    usleep(100);
    Q_EMIT closeOk();
}



void Backbone::translationReady() {
    bool changed = 0; // prevents from doubling ready() signal, when both if's are
                      //  executed in one translationReady() call then second
                      // translationReady() call doubles ready*() emit

    if(!dictFin && _innerResult.isFinished()) {
        changed = 1;
        dictFin = 1;
        QFutureIterator<QList<Translation*> > it(_innerResult);

        while(it.hasNext()) {
            QList<Translation* > list = it.next();
            foreach(Translation* trans, list) {
                if(!trans)
                    continue;
                if(!_searchBookmarks)
                    trans->setBookmark(_bookmarks.
                            inBookmarks(trans->key()));
                _result.insert(trans->key().toLower(), trans);
           }
        }
    }

    if(!bookmarkFin && _innerBookmarks.isFinished()) {
        changed = 1;
        bookmarkFin = 1;
        QList<Translation*> list = _innerBookmarks.result();

        foreach(Translation* trans, list)
                _result.insert(trans->key().toLower(), trans);
    }

    if(!stopped && bookmarkFin && dictFin && changed) {
        Q_EMIT ready();
    }
}




QStringList Backbone::getFilesFromDir(QString dir, QStringList nameFilter) {
    QDir plug(QDir::toNativeSeparators(dir));
    if(!plug.exists()) {
        qDebug() << plug.absolutePath() << " folder doesn't exist";
        Q_EMIT notify(Notify::Warning,
                tr("%1 folder doesn't exist.").arg(plug.path()));
        return QStringList();
    }
    plug.setFilter(QDir::Files);
    QStringList list = plug.entryList(nameFilter);

    for(int i = 0; i < list.size(); i++)
        list[i] = plug.absoluteFilePath(list.at(i));
    return list;
}


void Backbone::loadPlugins() {
    if(dryRun)
        return;
    QStringList nameFilter;
    nameFilter << "*.so" << "*.so.*";
    QStringList files = getFilesFromDir(_pluginPath, nameFilter);

    foreach(QString file, files) {
        QPluginLoader loader(file);
        if(!loader.load()) {
            Q_EMIT notify(Notify::Error,
                    tr("%1 plugin cannot be loaded: %2.")
                    .arg(file).arg(loader.errorString()));
            continue;
        }
        QObject *pl = loader.instance();

        bool exists = 0;
        CommonDictInterface *plugin = qobject_cast<CommonDictInterface*>(pl);
        foreach(CommonDictInterface* pl, _plugins)
            if(pl->type() == plugin->type()) {
                exists = 1;
                break;
           }
        if(!exists) {
            _plugins.append(plugin);
            plugin->retranslate();
            connect(plugin, SIGNAL(notify(Notify::NotifyType,QString)),
                    this, SIGNAL(notify(Notify::NotifyType,QString)));
        }
    }
}



CommonDictInterface* Backbone::plugin(QString type) {
    foreach(CommonDictInterface* plugin, _plugins)
        if(plugin->type() == type)
            return plugin;
    return 0;
}



void Backbone::loadPrefs(QString fileName) {
    if(dryRun)
        return;
    QFileInfo file(QDir::toNativeSeparators(fileName));
    QDir confDir(file.dir());
    if(!confDir.exists()){
        qDebug() << "Configuration file doesn't exist ("
                << file.filePath() << ")";
        Q_EMIT notify(Notify::Warning,
                tr("%1 configuration file doesn't exist.")
                .arg(file.filePath()));
        return;
    }
    QSettings set(file.filePath(), QSettings::IniFormat);
    _pluginPath = set.value("general/plugin_path", _pluginPath).toString();
    _historyLen = set.value("general/history_size", 10).toInt();
    _searchLimit = set.value("general/search_limit", 15).toInt();
    _searchBookmarks = set.value("general/search_bookmarks",1).toBool();
    _searchDicts = set.value("general/search_dictionaries",1).toBool();
    _zoom = set.value("general/zoom", 1.0).toReal();
}



void Backbone::savePrefs(QSettings *set) {
    if(dryRun)
        return;
    set->setValue("general/plugin_path", _pluginPath);
    set->setValue("general/history_size", _historyLen);
    set->setValue("general/search_limit", _searchLimit);
    set->setValue("general/search_bookmarks", _searchBookmarks);
    set->setValue("general/search_dictionaries", _searchDicts);
    set->setValue("general/zoom", _zoom);
}



void Backbone::loadDicts(QString fileName) {
    if(dryRun)
        return;

    QFileInfo file(QDir::toNativeSeparators(fileName));
    QDir confDir(file.dir());
    if(!confDir.exists()){
        qDebug() << "Configuration file doesn't exist ("
                << file.filePath() << ")";
        Q_EMIT notify(Notify::Warning,
                tr("%1 configuration file doesn't exist.")
                .arg(file.filePath()));
        return;
    }

    QSettings set(file.filePath(), QSettings::IniFormat);
    QStringList dicts = set.childGroups();
    foreach(QString dict, dicts) {
        if(!dict.contains("dictionary_"))
            continue;
        CommonDictInterface* plug = plugin
                                    (set.value(dict + "/type", "").toString());
        if(!plug) {
            qDebug() << "Config file error: "
                    << set.value(dict + "/type", "").toString()
                    << " doesn't exist";
            Q_EMIT notify(Notify::Warning,
                    tr("Configuration file error. %2 plugin doesn't exist.")
                    .arg(set.value(dict + "/type", "").toString()));
            continue;
        }
        Settings* plugSet = new Settings();
        set.beginGroup(dict);
        QStringList items = set.childKeys();
        foreach(QString item, items) {
            plugSet->setValue(item, set.value(item, "").toString());
        }
        bool active = set.value("active",1).toBool();

        set.endGroup();
        addInternalDictionary(plug->getNew(plugSet), active);
        delete plugSet;
    }
}



void Backbone::dictUpdated() {
    if(dryRun)
        return;

    // For convienence this function is called for each change in dictionaries
    // and each call dumps configuration for all dictionaries into file.
    // Maybe a better way would be to store new/changed configuration but
    // parsing settings file and figuring out what was changed, in my opinion,
    // would take more time
    _history->setMaxSize(_historyLen);
    QFileInfo file(QDir::toNativeSeparators(_configPath));
    QDir confDir(file.dir());
    if(!confDir.exists())
        confDir.mkpath(file.dir().path());
    QSettings set(file.filePath(), QSettings::IniFormat);
    set.clear();

    savePrefs(&set);

    foreach(CommonDictInterface* dict, _dicts.keys()){
        if(!dict || !dict->settings())
            continue;
        saveState(&set, dict->settings(), _dicts[dict], dict->hash());
    }
}



void Backbone::saveState(QSettings* set, Settings* plugSet, bool active
                         , uint hash) {
    if(dryRun)
        return;
    if(!set || !plugSet)
        return;

    QString section;
    section.append(QString("dictionary_%1").arg(hash));
    QList<QString> keys = plugSet->keys();
    foreach(QString key, keys)
        set->setValue(section + "/" + key, plugSet->value(key));
    set->setValue(section + "/active", active);
}



QStringList Backbone::xmls() {
    return _xmlResult;
}

void Backbone::addBookmark(QList<Translation*> translations) {
    if(translations.size()>0) {
        Translation *tr = translations.at(0);
        foreach(CommonDictInterface* dict, activeDicts()) {
            Translation* trans = dict->getTranslationFor(tr->key());
            if(trans) {
                translations.append(trans);
            }
        }
    }

    Translation* translation;
    QHash<QString,QString> trans;
    foreach(translation,translations){
        translation->setBookmark(true);
        trans.insert(translation->toXml(),translation->key());

    }

    foreach(QString value,trans.keys()){
        QString key= trans.value(value);
        _bookmarks.add(key,key,value);
    }

    // Translation* translation;
    // foreach(translation, translations)
    // _bookmarks.add(translation);
    // emit bookmarkReady();
}


void Backbone::searchXml(QList<Translation *> translations) {
    _xmlResult.clear();

    QList<TranslationPtr> dummy;
    stopped = false;
    foreach(Translation* tr, translations) {
         if(containsDict(tr->dict()) || !tr->dict())
            dummy.append(TranslationPtr(tr));
    }
    if(translations.size()>0) {
        Translation *tr = translations.at(0);
        foreach(CommonDictInterface* dict, activeDicts()) {
            Translation* trans = dict->getTranslationFor(tr->key());
            if(trans)
                dummy.append(TranslationPtr(trans));
        }
    }

   _innerXmlResult = QtConcurrent::mapped(dummy,
                                            &TranslationPtr::toHtml);
   _xmlResultWatcher.setFuture(_innerXmlResult);
}



void Backbone::xmlTranslationReady() {

    QFutureIterator<QString> it(_innerXmlResult);
    QSet<QString> uniqe;
    while(it.hasNext())
        uniqe.insert(it.next());
    _xmlResult.clear();
    _xmlResult = uniqe.toList();

    if(!stopped)
        Q_EMIT xmlReady();

}


QList<CommonDictInterface*> Backbone::activeDicts() {
    QList<CommonDictInterface*>res;
    foreach(CommonDictInterface* dict, _dicts.keys())
        if(_dicts[dict])
            res.append(dict);
    return res;

}



void Backbone::bookmarksListReady() {
   _bookmarksResult = _innerBookmarks.result();
   Q_EMIT bookmarksReady();
}




void Backbone::setSettings(Settings *settings) {
    _historyLen = settings->value("history_size").toInt();
    _searchLimit = settings->value("search_limit").toInt();
    if(settings->value("search_dictionaries") == "true")
        _searchDicts = 1;
    else
        _searchDicts = 0;
    if(settings->value("search_bookmarks") == "true")
        _searchBookmarks = 1;
    else
        _searchBookmarks = 0;
    _zoom = settings->value("zoom").toFloat();
    if(!_zoom)
        _zoom ++;

    dictUpdated();
    if(settings)
        delete settings;
}




Settings* Backbone::settings() {
    Settings * settings = new Settings();
    settings->setValue("history_size", QString("%1").arg(_historyLen));
    settings->setValue("search_limit", QString("%1").arg(_searchLimit));
    settings->setValue("zoom", QString("%1").arg(_zoom));
    if(_searchBookmarks)
        settings->setValue("search_bookmarks", "true");
    else
        settings->setValue("search_bookmarks", "false");

    if(_searchDicts)
        settings->setValue("search_dictionaries", "true");
    else
        settings->setValue("search_dictionaries", "false");
    return settings;
}


bool Backbone::containsDict(uint hash) const {
    QHashIterator<CommonDictInterface*, bool> it(_dicts);
    if (!hash)
        return false;
    while(it.hasNext())
        if(it.next().key()->hash() == hash)
            return true;
    return false;
}
