/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/
/*!
    \file StarDialog.cpp
    \brief Implementation of stardict plugin's dialogs.

    \author Mateusz Półrola <mateusz.polrola@gmail.com>
*/

#include "StarDialog.h"
#include <QDebug>
#include <QFile>

StarDialog::StarDialog(StarDictPlugin *plugin, StarDialogType type,
                       QWidget *parent) : QDialog(parent) {
   this->plugin = plugin;
   this->type = type;
   initializeUI();
   if(type == New)
       connect(browseButton, SIGNAL(clicked()),this, SLOT(selectFile()));
   connect(confirmButton, SIGNAL(clicked()),this, SLOT(accept()));
}


void StarDialog::initializeUI() {
    mainVerticalLayout = new QVBoxLayout;
    widget = new QWidget;
    widget->setLayout(mainVerticalLayout);
    infoLabel = new QLabel;
    infoLabel->setWordWrap(true);
    QVBoxLayout* buttonLayout = new QVBoxLayout;

    if(type == New) {
        browseLayout = new QHBoxLayout;
        browseButton = new QPushButton(tr("Browse"));
        browseButton->setMaximumWidth(150);
        infoLabel->setText(tr("Dictionary file: not selected"));
        setWindowTitle(tr("Add new StarDict dictionary"));
        infoLabel->setText(tr("Dictionary file: not selected"));
        browseLayout->addWidget(infoLabel, 0, Qt::AlignLeft);
        browseLayout->addLayout(buttonLayout);
        browseLayout->addWidget(browseButton, 0, Qt::AlignRight);
        mainVerticalLayout->addLayout(browseLayout);
    }
    else {
        setWindowTitle(tr("StarDict Settings"));
        infoLabel->setText(tr("Plugin: ") + plugin->type().toUpper() +"\n" +
                tr("Book name: ") + plugin->settings()->value("bookname") 
                        + "\n" +
                tr("Version: ") + plugin->settings()->value("version") + "\n" +
                tr("Word count: ") + plugin->settings()->value("wordcount") 
                        + "\n" +
                tr("Author: ") + plugin->settings()->value("author") + "\n" +
                tr("E-mail: ") + plugin->settings()->value("email") + "\n" +
                tr("Website: ") + plugin->settings()->value("website") + "\n" +
                tr("Description: ") + plugin->settings()->value("description") 
                        + "\n" +
               tr("Date: ") + plugin->settings()->value("date"));
        mainVerticalLayout->addWidget(infoLabel);
    }

    if(!plugin)
        _dictionaryFilePath = "";

    confirmButton = new QPushButton;
    mainVerticalLayout->addWidget(confirmButton);
    if(type == New)
        confirmButton->setText(tr("Add"));
    else
        confirmButton->setText(tr("Save settings"));

    scrollArea = new QScrollArea;
    scrollArea->setWidget(widget);
    scrollArea->setWidgetResizable(true);
    #ifdef Q_WS_MAEMO_5
        scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    #else
        if(type==New) {
            infoLabel->setMinimumWidth(200);
//            setMinimumSize(sizeHint().width()*1.5, sizeHint().height()*1.2);
//            setMaximumSize(sizeHint().width()*1.7, sizeHint().height()*1.5);
            scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        }
    #endif

    layout = new QHBoxLayout;
    layout->addWidget(scrollArea);
    setLayout(layout);

    #ifndef Q_WS_MAEMO_5
        setMinimumSize(400,200);
    #else
        setMinimumHeight(350);
    #endif

    scrollArea->setLineWidth(0);
    scrollArea->setMidLineWidth(0);
    scrollArea->setFrameStyle(QFrame::NoFrame);
}


void StarDialog::selectFile() {
    QString fileName = QFileDialog::getOpenFileName(this,
                     tr("Select dictionary file"),
                     _dictionaryFilePath,
                     tr("StarDict Files (*.dict *dict.dz *.idx *idx.gz *.ifo)"),
                     NULL,
                     NULL);
    if (!fileName.isEmpty()) {
        infoLabel->setText(tr("Dictionary file: %1").arg(fileName));
        _dictionaryFilePath = fileName;
        if (_dictionaryFilePath.endsWith(".tar.bz2"))
            _isCompressed = true;
        else
            _isCompressed = false;
        updateGeometry();
    }
}


void StarDialog::saveSettings() {
    _settings = new Settings;

    if(plugin)
        foreach(QString key, plugin->settings()->keys())
            _settings->setValue(key, plugin->settings()->value(key));

    if(_settings->value("path")=="")
        _settings->setValue("path", _dictionaryFilePath);

    if(_settings->value("ifoFileName")=="")
        _settings->setValue("ifoFileName", _dictName + ".ifo");

    if(_settings->value("idxFileName")=="")
        _settings->setValue("idxFileName", _dictName + ".idx");

    if(_settings->value("dictFileName")==""){
        if (QFile::exists(_dictName + ".dict.dz") == true)
            _settings->setValue("dictFileName", _dictName + ".dict.dz");
        else
            _settings->setValue("dictFileName", _dictName + ".dict");
    }

    if(_settings->value("synFileName")=="")
        if (QFile::exists(_dictName + ".syn") == true)
            _settings->setValue("synFileName", _dictName + ".syn");
}


void StarDialog::accept() {
    if(type == New && _dictionaryFilePath.isEmpty()) {
        Q_EMIT notify(Notify::Warning, tr("File path is not set"));
        return;
    }

    if(type == New && !_dictionaryFilePath.isEmpty() && !checkFiles()) {
        Q_EMIT notify(Notify::Warning, tr("Dictionary files are not complete"));
        return;
    }

    saveSettings();
    QDialog::accept();
}


bool StarDialog::checkFiles() {
    if (!_isCompressed) {
        if (_dictionaryFilePath.right(2) == "dz") {
            _dictName = _dictionaryFilePath.left(_dictionaryFilePath.lastIndexOf("."));
            _dictName = _dictName.left(_dictName.lastIndexOf("."));
        }
        else{
            _dictName = _dictionaryFilePath.left(_dictionaryFilePath.lastIndexOf("."));
        }

        if (QFile::exists(_dictName + ".idx") == false
                && QFile::exists(_dictName + ".idx.gz") == false) {
            return false;
        }
        if (QFile::exists(_dictName + ".dict") == false
                && QFile::exists(_dictName + ".dict.dz") == false) {
            return false;
        }
        return true;
    }
    else
        return false;
}


Settings* StarDialog::getSettings() {
    return _settings;
}

